module Quandl::Cassandra::Base::Connection
  
  extend ActiveSupport::Concern
  
  module ClassMethods
    
    def truncate
      execute("TRUNCATE #{table_name}")
    end
    
    def delete(*ids)
      statement = prepare("DELETE FROM #{table_name} WHERE id = ?")
      Array(ids).flatten.
        collect{|id| statement.async.execute(id, consistency) }.
        collect(&:value)
    end
    
    def prepare(statement)
      with_connection do |c|
        c.prepare(statement)
      end
    rescue Cql::QueryError
      raise $!, "#{$!}\n  statement: #{statement}", $!.backtrace
    end
    
    def execute_async(statement, consist = nil)
      consist = consistency unless consist.present?
      with_connection do |c|
        c.async.execute( statement, consist )
      end
    rescue Cql::QueryError
      raise $!, "#{$!}\n  statement: #{statement}", $!.backtrace
    end
    
    def execute(statement, consist = nil)
      consist = consistency unless consist.present?
      with_connection do |c|
        c.execute( statement, consist )
      end
    rescue Cql::QueryError
      raise $!, "#{$!}\n  statement: #{statement}", $!.backtrace
    end
    
    def with_connection(&block)
      rescue_and_retry(2) do
        block.call(connection)
      end
    end
    
    def rescue_and_retry(times, &block)

      block.call(connection)
      
    rescue Cql::QueryError => err
      #puts "ERROR: #{err.code} #{err}\n  cql: '#{err.cql}'"
      Quandl::Logger.error("#{err.code} '#{err.cql}' #{$!}")
      # raise error if it's failed three times
      raise( $!, "#{err.code} '#{err.cql}' #{$!}", $!.backtrace ) if times <= 1
      # otherwise rescue and retry
      rescue_and_retry(times - 1, &block)
      
    rescue Cql::Io::ConnectionError, Cql::NotConnectedError => err
      #puts "ERROR: #{err}"
      Quandl::Logger.error(err)
      reset_connection
      raise if times <= 1
      rescue_and_retry(times - 1, &block)
      
    end
    
    def connection
      @@connection ||= establish_connection
    end
    
    def consistency
      Quandl::Cassandra.configuration.consistency
    end
    
    def reset_connection
      connection.close
      @@connection = establish_connection
      true
    end
    
    def establish_connection
      c = Cql::Client.connect(
        hosts:        Quandl::Cassandra.configuration.hosts, 
        consistency:  Quandl::Cassandra.configuration.consistency )
      # switch keyspace
      c.use( Quandl::Cassandra.configuration.keyspace ) if Quandl::Cassandra.configuration.keyspace.present?
      @@connection = c
      c
    end
    
  end  
  
end
