module Quandl::Cassandra::Base::Schema
  extend ActiveSupport::Concern

  def primary_key?
    !primary_key_attributes.values.include?(nil)
  end

  def primary_key_attributes
    self.class.primary_key.inject({}) { |m,key| m[key] = self.send(key); m }
  end

  module ClassMethods
    def initialize_class_once
      return if defined?(@initialize_class_once)
      define_schema_attributes
      @initialize_class_once = true
    end

    def column_type(type)
      column_types[type.to_sym]
    end

    def column_types
      unless @column_types && @column_types.is_a?(Hash) && !@column_types.empty?
        @column_types = {}
        columns.each do |c|
          @column_types[c.column_name.to_sym] = Quandl::Cassandra::Types.type(c.type)
        end
        Quandl::Logger.info("Column types for #{self.to_s}: #{@column_types}")
      end
      @column_types
    end

    def column_names
      unless @column_names && @column_names.is_a?(Array) && !@column_names.empty?
        @column_names = columns.map(&:column_name)
        Quandl::Logger.info("Column names for #{self.to_s}: #{@column_names.join(', ')}")
      end
      @column_names
    end

    def columns
      unless @columns && @columns.is_a?(Cql::Client::ResultMetadata)
        @columns = execute("SELECT * FROM #{table_name} LIMIT 1").metadata
        Quandl::Logger.info("Columns for #{self.to_s} #{@columns.inspect}")
      end
      @columns
    end

    def primary_key
      @primary_key ||= key_aliases + column_aliases
    end

    def column_aliases
      return [] if system_schema.blank?
      @column_aliases ||= JSON.parse(system_schema['column_aliases']).map(&:to_sym)
    rescue JSON::ParserError
      []
    end

    def key_aliases
      return [] if system_schema.blank?
      @key_aliases ||= JSON.parse(system_schema['key_aliases']).map(&:to_sym)
    rescue JSON::ParserError
      []
    end

    def table_exists?
      system_schema.present?
    end

    def system_schema
      return @system_schema if defined?(@system_schema)
      @system_schema = execute(%Q{
        SELECT * FROM system.schema_columnfamilies WHERE
        keyspace_name='#{Quandl::Cassandra.configuration.keyspace}'
        AND columnfamily_name = '#{table_name}'
        }, 'ONE').first
    end

    private

    def define_schema_attributes
      self.class_eval do
        define_attributes *column_names
      end
    end
  end

  def initialize(*args, &block)
    self.class.initialize_class_once
    super if defined?(super)
  end
end
