module Quandl::Cassandra::Base::Scoping
  extend ActiveSupport::Concern

  module ClassMethods
    def scope_names
      scope.scope_names
    end

    def where(*args)
      scope.new.where(*args)
    end
  end

  included do
    include ScopeComposer::Model

    has_scope_composer

    scope :select, :delete_all, :limit,
      :order, :count, :find, :first, :all, :pluck, :exists?, :to_h

    scope.class_eval do

      attr_accessor :statement_type

      delegate :to_a, to: :execute

      delegate :primary_key, :table_name, :sanitize_attributes, to: :parent

      include Enumerable

      def each(&block)
        all.each do |record|
          if block_given?
            block.call(record)
          else
            yield record
          end
        end
      end

      def exists?
        columns = primary_key.collect{|pk| attributes.keys.include?(pk) ? pk : nil }.compact
        select(*columns).execute.present?
      end

      def to_h
        execute
      end

      def pluck(*args)
        r = select(*args).values
        r = r.flatten if args.count <= 1
        r
      end

      def select=(*names)
        self.statement_type = :select
        write_scope_attribute(:select, names)
      end

      def values
        to_a.collect(&:values)
      end

      def find(*args)
        primary_key = parent.bind_primary_key(*args)
        return nil if primary_key.blank?
        where(primary_key).first
      end

      def first
        limit(1).execute_returning_record
      end

      def all
        execute_returning_collection
      end

      def count
        select("COUNT(*)").execute.first.try(:[], 'count').to_i
      end

      def to_cql
        [cql_statement_type, table_name, cql_where, cql_order, cql_limit].compact.join(' ')
      end

      def delete_all
        self.statement_type = :delete
        execute
      end

      def insert
        self.statement_type = :insert
        self
      end

      protected

      def cql_statement_type
        case statement_type
        when :insert then "INSERT INTO"
        when :delete then "DELETE FROM"
        else
          "SELECT #{cql_select} FROM"
        end
      end

      def cql_where
        attrs = attributes.clone
        attrs = sanitize_attributes(attrs) if parent.table_exists?
        attrs = attrs.collect{|key, value| cql_where_attribute(key, value) }.join(' AND ')
        cql = attrs.present? ? "WHERE #{attrs}" : nil
      end

      def cql_where_attribute(key, value)
        if value.is_a?(Array)
          "#{key} IN (#{value.join(',')})"
        else
          "#{key} = #{value}"
        end
      end

      def cql_select
        columns = Array(select).join(',') if select.present?
        columns = '*' if columns.blank?
        columns
      end

      def cql_order
        "ORDER BY #{order}" if order.present?
      end

      def cql_limit
        "LIMIT #{limit}" if limit.present?
      end

      def execute
        @execute ||= parent.execute(to_cql)
      end

      def execute_async
        @execute_async ||= parent.execute(to_cql)
      end

      def parent
        self.class.parent
      end

      def execute_returning_collection
        execute.collect{|r| parent.new_from_query_result(r) }
      end

      def execute_returning_record
        parent.new_from_query_result( execute.first )
      end

    end

  end

end